/*!
   ZTEX Firmware Kit for EZ-USB FX3 Microcontrollers
   Copyright (C) 2009-2014 ZTEX GmbH.
   http://www.ztex.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!*/
/*
    Example implementation of SD card as secondary flash device.
*/    

#include "cyu3system.h"
#include "cyu3os.h"
#include "cyu3usb.h"
#include "cyu3dma.h"
#include "cyu3error.h"
#include "cyu3sib.h"

#define FLASH_EC_TIMEOUT 2
#define FLASH_EC_BUSY 3
#define FLASH_EC_PENDING 4
#define FLASH_EC_NOTSUPPORTED 7
#define FLASH_EC_RUNTIME 8

struct __attribute__((__packed__)) ztex_flash2_t {
    uint8_t enabled;			// 0	1: enabled, 0:disabled
    uint16_t sector_size; 		// 1    sector size <sector size> = MSB==0 : flash2_sector_size and 0x7fff ? 1<<(flash2_sector_size and 0x7fff)
    uint32_t sectors;			// 3	number of sectors
    uint8_t ec; 	       		// 7	error code
    
    uint32_t ep0_read_addr;
    uint8_t need_pp;
    uint32_t write_addr;
};

struct ztex_flash2_t ztex_flash2;

// VR 0x40
uint8_t vr_flash2_info(uint16_t value, uint16_t index, uint16_t length ) {
    CyU3PMemCopy( ztex_ep0buf, (uint8_t*)&ztex_flash2, 8);
    ZTEX_REC_RET( CyU3PUsbSendEP0Data( 8, ztex_ep0buf ) );
    return 0;
}

// VR 0x41
uint8_t vr_flash2_read(uint16_t value, uint16_t index, uint16_t length ) {
    index = index >> 8;
    if ( index==0 ) {
//        ZTEX_LOG("FR: %d, %d", index, value);
	ztex_flash2.ep0_read_addr = value << 16;
    }
//    if ( ztex_flash2_read(ztex_ep0buf,ztex_flash2.ep0_read_addr,length) ) return 255;
    ztex_flash2.ep0_read_addr += length;
    ZTEX_REC_RET ( CyU3PUsbSendEP0Data( length, ztex_ep0buf ) );
    return 0;
}

// VC 0x42
uint8_t vc_flash2_write(uint16_t value, uint16_t index, uint16_t length ) {
    index = index >> 8;
//    ZTEX_LOG("FW: %d, %d", index, value);
//    if ( index==0 && ztex_flash2_write_start(value) ) return 255;

    ZTEX_REC_RET ( CyU3PUsbGetEP0Data (length, ztex_ep0buf, NULL) );
//    if ( ztex_flash2_write(ztex_ep0buf,length) ) return 255;
    
//    if ( index==3 ) ztex_flash2_write_finish();
    return 0;
}

void ztex_flash2_init () {
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PSibIntfParams_t intfParams;
    CyU3PSibDevInfo_t devInfo;

    ztex_flash2.enabled = 0;
    ztex_flash2.ec = FLASH_EC_NOTSUPPORTED;

    ztex_register_vendor_req(0x44, vr_flash2_info);
    ztex_register_vendor_req(0x45, vr_flash2_read);
    ztex_register_vendor_cmd(0x46, vc_flash2_write);

    // Configure SD card port
    intfParams.cardDetType = CY_U3P_SIB_DETECT_DAT_3;   // Card detect based on SD_DAT[3]
    intfParams.writeProtEnable = CyFalse;               // Write protect disabled
    intfParams.useDdr = CyTrue;                         // DDR clocking enabled
    intfParams.maxFreq = CY_U3P_SIB_FREQ_104MHZ;        // No S port clock limitation
    intfParams.cardInitDelay   = 0;                     // No SD/MMC initialization delay required
    intfParams.resetGpio = 0xFF;			// No reset / low voltage switch GPIO's
    intfParams.rstActHigh = CyTrue; 
    intfParams.voltageSwGpio = 0xFF;
    intfParams.lvGpioState = CyFalse;
    intfParams.lowVoltage = CyFalse;
    ZTEX_REC( status = CyU3PSibSetIntfParams (0, &intfParams) );
    if (status != CY_U3P_SUCCESS) goto errreturn;
    
    // start the module and initialize SD card
    ZTEX_REC( status = CyU3PSibStart() );
    if (status != CY_U3P_SUCCESS) goto errreturn;
    
    // read SD card properties
    ZTEX_REC( status = CyU3PSibQueryDevice (0, &devInfo) );
    if (status != CY_U3P_SUCCESS) goto errreturn;
    
    if ( (devInfo.numUnits == 0) || ( (devInfo.cardType != CY_U3P_SIB_DEV_SD) && (devInfo.cardType != CY_U3P_SIB_DEV_MMC)) ) goto errreturn;
    
    ztex_flash2.enabled = 1;
    ztex_flash2.sector_size = devInfo.blkLen;
    ztex_flash2.sectors = devInfo.numBlks;

    
    ZTEX_LOG("Found %d MByte SD Flash",(ztex_flash2.sectors>>14)*(ztex_flash2.sector_size>>6));
    return;
    
errreturn:
    ztex_flash2.ec = FLASH_EC_RUNTIME;
    ztex_log("No SD Flash found");
}

void ztex_usb_stop_flash2() {
}
